import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Modal, Button } from 'antd'
import { SettingOutlined } from '@ant-design/icons'

import SettingForm from './settingform'

// import './index.scss'

class SettingComponent extends Component {
  static propTpyes = {
    mainsearch: PropTypes.any,       // 主表的搜索条件，当子表设置接收主表条件时有效
    MenuID: PropTypes.string,        // 菜单ID
    config: PropTypes.object,        // 菜单配置信息
    updatesetting: PropTypes.func
  }

  state = {
    menu: null,          // 菜单信息
    search: null,        // 搜索条件，包括主表搜索
    formlist: null,      // 表单信息
    visible: false,      // 模态框控制
    loading: false,      // 设置信息验证保存中
    record: null
    
  }

  /**
   * @description 全局设置触发
   */
  changeSetting = () => {
    const { MenuID, config, mainsearch } = this.props
    let menu = { MenuID: MenuID, MenuName: config.MenuName || config.tabName || '', MenuNo: config.MenuNo || config.tabNo || '' }

    let _search = fromJS(config.search).toJS()

    if (mainsearch) { // 综合主页搜索及子表搜索条件
      _search = [...mainsearch, ..._search]
    }

    this.setState({
      visible: true,
      search: _search,
      menu: menu
    })
  }

  /**
   * @description 保存页面配置信息
   */
  settingSave = () => {
    const { config } = this.props

    this.setState({
      loading: true
    })
    this.settingRef.handleConfirm().then(setting => {
      let res = this.resetSetting(setting, config.setting)
      this.setState({
        visible: false,
        loading: false
      })

      this.props.updatesetting({...config, setting: res})
    }, () => {
      this.setState({
        loading: false
      })
    })
  }

  // /**
  //  * @description 创建表格接口（读出）
  //  */
  // tableCreatInterface = () => {
  //   const { config } = this.props
  //   const { menu } = this.state

  //   this.settingRef.handleConfirm('loading').then(setting => {
  //     let res = this.resetSetting(setting, config.setting)
  //     let _config = {...config, setting: res}
  //     let _menu = {
  //       type: config.Template === 'CommonTable' ? 'main' : 'subtable',
  //       MenuID: menu.MenuID,
  //       menuName: menu.MenuName,
  //       menuNo: menu.MenuNo
  //     }

  //     this.refs.tableCreatInterface.triggerOutInterface(_menu, _config)
  //   })
  // }

  resetSetting = (s, ori) => {
    let setting = fromJS(s).toJS()
    // let maxScript = 0

    setting.show = ori.show || 'true'
    setting.advanceType = ori.advanceType || 'modal'
    setting.advanceWidth = ori.advanceWidth || 1000
    setting.drawerPlacement = ori.drawerPlacement || 'right'
    setting.searchRatio = ori.searchRatio || 6
    setting.searchLwidth = ori.searchLwidth !== undefined ? ori.searchLwidth : 33.3
    setting.resetContrl = ori.resetContrl || 'init'

    // if (window.GLOB.funcs && window.GLOB.funcs.length > 0) {
    //   window.GLOB.funcs.forEach(m => {
    //     let reg = new RegExp('\\$ex@' + m.func_code + '@ex\\$', 'ig')
    //     setting.scripts.forEach(item => {
    //       item.sql = item.sql.replace(reg, `/*$ex@${m.func_code}-begin*/\n${m.key_sql}\n/*@ex$-end*/`)

    //       if (item.status === 'false') return

    //       if (/exec\s/ig.test(item.sql)) {
    //         maxScript = 1000
    //       } else if (item.sql.length > maxScript) {
    //         maxScript = item.sql.length
    //       }
    //     })
    //     setting.preScripts.forEach(item => {
    //       item.sql = item.sql.replace(reg, `/*$ex@${m.func_code}-begin*/\n${m.key_sql}\n/*@ex$-end*/`)
    //     })
    //     setting.cbScripts.forEach(item => {
    //       item.sql = item.sql.replace(reg, `/*$ex@${m.func_code}-begin*/\n${m.key_sql}\n/*@ex$-end*/`)
    //     })
    //     if (setting.dataresource) {
    //       setting.dataresource = setting.dataresource.replace(reg, `/*$ex@${m.func_code}-begin*/\n${m.key_sql}\n/*@ex$-end*/`)
    //     }
    //   })
    // } else {
    //   setting.scripts.forEach(item => {
    //     if (item.status === 'false') return

    //     if (/exec\s/ig.test(item.sql)) {
    //       maxScript = 1000
    //     } else if (item.sql.length > maxScript) {
    //       maxScript = item.sql.length
    //     }
    //   })
    // }

    // setting.maxScript = maxScript

    return setting
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
  }

  render() {
    const { config } = this.props
    const { visible, search, menu } = this.state

    return (
      <>
        <SettingOutlined style={{position: 'absolute', fontSize: '18px', right: '7px', top: '5px', padding: '10px', zIndex: 1}} onClick={this.changeSetting} />
        {/* 设置全局配置及列表数据源 */}
        <Modal
          wrapClassName="mk-pop-modal"
          visible={visible}
          width={'75vw'}
          maskClosable={false}
          onCancel={() => { this.setState({ visible: false, loading: false })}}
          footer={[
            <Button key="cancel" onClick={() => { this.setState({ visible: false, loading: false }) }}>取消</Button>,
            <Button key="confirm" type="primary" loading={this.state.loading} onClick={this.settingSave}>确定</Button>
          ]}
          destroyOnClose
        >
          <SettingForm
            menu={menu}
            config={config}
            search={search}
            updRecord={() => {}}
            wrappedComponentRef={(inst) => this.settingRef = inst}
          />
        </Modal>
      </>
    )
  }
}

export default SettingComponent